
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.util.*;


public class SimpleMenuApplet extends Applet
        implements MouseListener, ActionListener
{

  /**
   * Field image, font, fsize, data
   */
  String image, font, fsize, data;

  /**
   * Field menuFont
   */
  Font menuFont;

  /**
   * Field menus[]
   */
  Menu menus[];

  /**
   * Field menuItems[]
   */
  MenuItem menuItems[];

  /**
   * Field actions[]
   */
  String actions[];

  /**
   * Field targets[]
   */
  String targets[];

  /**
   * Field fontSize
   */
  int fontSize;

  /**
   * Field layout
   */
  FlowLayout layout;

  /**
   * Field panel
   */
  PopupMenu panel;

  /**
   * Field imgButton
   */
  Image imgButton = null;

  /**
   * Field parseError
   */
  boolean parseError = false;

  /**
   * Field imgError
   */
  boolean imgError = false;

  /**
   * Field imgUrl
   */
  URL imgUrl;

  /**
   * Constructor SimpleMenuApplet
   *
   *
   */
  public SimpleMenuApplet() {}

  /**
   * Method init
   *
   *
   *
   *
   *
   */
  public void init()
  {

    layout = new FlowLayout();
    font   = getParameter("font");

    if (font == null){
      font = "SansSerif";
    }

    fsize = getParameter("fsize");

    if (fsize == null){
      fontSize = 12;
    }
    else{
      try{
        fontSize = Integer.parseInt(fsize);
      }
      catch (NumberFormatException nfe){
        fontSize = 12;
      }
    }

    // Make applet params set menu font stuff
    menuFont = new Font(font, Font.BOLD, fontSize);

    // Now handle data
    data = getParameter("data");

    parseData(data);
    this.addMouseListener(this);
    this.setLayout(layout);

    // Make sure you can get an image
    image = getParameter("image");

    if (image != null){
      try{    // absolute URL
        imgUrl    = new URL(image);
        imgButton = this.getImage(imgUrl);
      }
      catch (MalformedURLException a){
        try{  // relative URL
          imgUrl    = new URL(this.getDocumentBase(), image);
          imgButton = this.getImage(imgUrl);
        }
        catch (MalformedURLException r){
          imgError = true;
        }
      }

      imgButton = this.getImage(this.getDocumentBase(), image);
    }
    else{
      imgError = true;
    }
  }

  /**
   * Method start
   *
   *
   *
   *
   *
   */
  public void start() {}

  /**
   * Method stop
   *
   *
   *
   *
   *
   */
  public void stop() {}

  /**
   * Method destroy
   *
   *
   *
   *
   *
   */
  public void destroy() {}

  /**
   * Method paint
   *
   *
   * @param g
   *
   *
   *
   *
   */
  public void paint(Graphics g)
  {

    if (imgButton != null){
      g.drawImage(imgButton, 0, 0, this);
    }

    if (imgError){
      this.showStatus("SimpleMenuApplet: BAD IMAGE " + image);
    }

    if (parseError){
      this.showStatus("SimpleMenuApplet: BAD DATA");
    }
  }

  /**
   * Method mousePressed
   *
   *
   * @param e
   *
   *
   *
   *
   */
  public void mousePressed(MouseEvent e)
  {
    popup();
  }

  /**
   * Method mouseReleased
   *
   *
   * @param e
   *
   *
   *
   *
   */
  public void mouseReleased(MouseEvent e) {}

  /**
   * Method mouseEntered
   *
   *
   * @param e
   *
   *
   *
   *
   */
  public void mouseEntered(MouseEvent e) {}

  /**
   * Method mouseExited
   *
   *
   * @param e
   *
   *
   *
   *
   */
  public void mouseExited(MouseEvent e) {}

  /**
   * Method mouseClicked
   *
   *
   * @param e
   *
   *
   *
   *
   */
  public void mouseClicked(MouseEvent e) {}

  /**
   * Method actionPerformed
   *
   *
   * @param ae
   *
   *
   *
   *
   */
  public void actionPerformed(ActionEvent ae)
  {

    int    curMenu = Integer.parseInt(ae.getActionCommand());
    String surfTo  = actions[curMenu];
    String tgt     = targets[curMenu];

    try{

      // Make surfTo a  URL
      URL url = new URL(this.getDocumentBase(), surfTo);

      this.getAppletContext().showDocument(url, tgt);
    }
    catch (MalformedURLException r){
      this.showStatus("SimpleMenuApplet: BAD URL " + surfTo);
    }
  }

  /**
   * Method popup
   *
   *
   *
   *
   *
   */
  public void popup()
  {

    if (!parseError){
      panel.show(this, 10, this.getSize().height);
    }
  }

  /**
   * Method parseData
   *
   *
   * @param data
   *
   *
   *
   *
   */
  private void parseData(String data)
  {

    // These guys are set to -1 so first call bumps to 0
    int levelCtr = -1, menuItemCount = -1, menuItemCtr = -1;

    // These guys are set to 0 so first call bumps to 1
    int     levelCount = 0, menuCount = 0, menuCtr = 0;
    int     parent[];
    String  itemToken = null;
    String  datatoken = null;
    String  title     = "";
    String  action    = "";
    String  target    = "_self";
    boolean newMenu   = false;

    if ((data == null) || (data.indexOf("{") == -1)){
      parseError = true;

      return;
    }

    StringTokenizer menuTag = new StringTokenizer(data, "{}", true);
    StringTokenizer itemTag = new StringTokenizer(data, "{}", true);
    StringTokenizer sep;

    do{
      try{
        itemToken = itemTag.nextToken();
      }
      catch (NoSuchElementException i) {}

      if (itemToken.charAt(0) == '{'){
        if (newMenu){
          menuCount++;
        }

        newMenu = true;

        levelCtr++;

        if (levelCount < levelCtr){
          levelCount = levelCtr;
        }
      }
      else if (itemToken.charAt(0) == '}'){
        if (newMenu){
          menuItemCount++;
        }

        newMenu = false;

        levelCtr--;
      }
    }
    while (itemTag.hasMoreTokens());

    if (levelCtr != -1){
      parseError = true;

      return;
    }

    // Bump by 1 since the first value is 0
    actions   = new String[menuItemCount + 1];
    targets   = new String[menuItemCount + 1];
    menuItems = new MenuItem[menuItemCount + 1];
    menus     = new Menu[menuCount + 1];
    parent    = new int[levelCount + 1];
    panel     = new PopupMenu(title);
    menus[0]  = (Menu)(panel);

    this.add(panel);

    itemToken = null;
    newMenu   = false;

    // Parse the data
    do{
      try{
        itemToken = menuTag.nextToken();
      }
      catch (NoSuchElementException i) {}

      if (itemToken.charAt(0) == '{'){
        if (newMenu){
          menuCtr++;

          menus[menuCtr] = new Menu(title);

          menus[menuCtr].setFont(menuFont);
          menus[parent[levelCtr]].add(menus[menuCtr]);

          parent[levelCtr + 1] = menuCtr;
        }

        newMenu = true;

        levelCtr++;
      }
      else if (itemToken.charAt(0) == '}'){
        if (newMenu){
          menuItemCtr++;

          actions[menuItemCtr]   = action;
          targets[menuItemCtr]   = target;
          menuItems[menuItemCtr] = new MenuItem(title);

          menuItems[menuItemCtr].setFont(menuFont);
          menuItems[menuItemCtr].addActionListener(this);
          menuItems[menuItemCtr]
            .setActionCommand(new Integer(menuItemCtr).toString());
          menus[parent[levelCtr]].add(menuItems[menuItemCtr]);
        }

        newMenu = false;

        levelCtr--;
      }
      else if (!itemToken.trim().equals("")){
        sep = new StringTokenizer(itemToken, "!");

        try{
          title = sep.nextToken();

          // Use - as Menu Separator
          if (title.equals("-")){
            title = "-";
          }
        }
        catch (NoSuchElementException i){

          // Got nothing
          title = "-";
        }

        try{
          action = sep.nextToken();
        }
        catch (NoSuchElementException i){

          // Got nothing
          action = "";
        }

        try{
          target = sep.nextToken();
        }
        catch (NoSuchElementException i){

          // Make it self unless declared
          target = "_self";
        }
      }
    }
    while (menuTag.hasMoreTokens());
  }
}

