package gallery;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import javax.servlet.ServletException;
import javax.servlet.RequestDispatcher;

import java.io.IOException;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

public class ControllerServlet extends HttpServlet {
    private static String INVENTORY_ROOT;

    private Map commands = new HashMap();
    private Inventory inventory = null;

    public void init() throws ServletException {
		// initialize path
		ServletConfig config = this.getServletConfig();
		ServletContext context = config.getServletContext();
		INVENTORY_ROOT = context.getRealPath("/images");
		
        // initialize inventory.
        inventory = new Inventory(INVENTORY_ROOT,
                                  "images/");

        // setup all commands.
        commands.put("GET_IMAGES", new GetImagesCommand());
    }

    public void doGet(HttpServletRequest req, HttpServletResponse res) 
        throws ServletException, IOException {
        doPost(req, res);
    }

    public void doPost(HttpServletRequest req, HttpServletResponse res) 
        throws ServletException, IOException {

        logmsg("doPost");
        // Fetch command-name from request
        String commandName = req.getParameter("COMMAND");

        // Fetch command from command-name
        Command c = (Command)commands.get(commandName);

        // handle command.
        c.handle(req, res);
    }


    private class GetImagesCommand implements Command {
        public void handle(HttpServletRequest req, HttpServletResponse res) 
            throws ServletException, IOException {
            logmsg("command->handle");

            String catalog = req.getParameter("CATALOG");
            String numRows = req.getParameter("ITEMS_PER_ROW");

            // Fetch images from catalog.
            Iterator images = inventory.getImagesFor(catalog);

            // Set data for view.
            req.setAttribute("IMAGES", images);
            req.setAttribute("ITEMS_PER_ROW", numRows);

            // Forward to the view.
            RequestDispatcher rd = req.getRequestDispatcher("/galleryViewer");
            rd.forward(req, res);
        }
    }

    private void logmsg(String s) {
        System.err.println("logmsg: " + s);
    }
}
