#
# Project: JSP: A Beginners Guide
# Author : Gary Bollinger and Bharathi Natarajan
# Date   : Jul 02 2001 13:34
#
#
# Target DBMS: 'MySQL'
#
 
 
CREATE DATABASE family_center;
 
#
# select the database
#
USE family_center;
 
 
#
# Table               : household
# Description         : Table of all households
#
# id                  : Uniquekey 
# name                : Family name 
# address             : Address text 
# phone               : Phone number 
#
CREATE TABLE household (
    id              INTEGER NOT NULL AUTO_INCREMENT,
    name            VARCHAR(100) NOT NULL,
    address         VARCHAR(255),
    phone           VARCHAR(20),
PRIMARY KEY (id)); 
 
#
# Table               : event
# Description         : Table of all household events
#
# id                  : Unique key 
# name                : Name of event 
# event_dt            : Date of event 
# album_dir           : Path to image directory 
# descr               : Description field 
# houdehod_id         : Key into household
#
CREATE TABLE event (
    id              INTEGER NOT NULL AUTO_INCREMENT,
    name            VARCHAR(100) NOT NULL,
    event_dt        DATE NOT NULL,
    album_dir       VARCHAR(255),
    descr           VARCHAR(255),
    household_id    INTEGER,
PRIMARY KEY (id),
FOREIGN KEY (household_id) REFERENCES household (id));
 
#
# Table               : messageboard
# Description         : Table of all posted messages
#
# id                  : Unique id 
# subject             : Theme of message 
# message_dt          : Date of message 
# message             : Actual text of message 
# member_id           : Key into member
#
CREATE TABLE messageboard (
    id              INTEGER NOT NULL AUTO_INCREMENT,
    message_dt      DATE,
    subject         VARCHAR(100),
    message         TEXT,
    member_id       INTEGER,
PRIMARY KEY (id),
FOREIGN KEY (member_id) REFERENCES member (id));
 
#
# Table               : member_role
# Description         : Table of all member roles
#
# id                  : Unique key 
# name                : Human readable name 
# description         : Specification for role 
#
CREATE TABLE MEMBER_ROLE (
    id              INTEGER NOT NULL AUTO_INCREMENT,
    name            VARCHAR(100) NOT NULL,
    description     VARCHAR(255),
PRIMARY KEY (id)); 

#
# Table               : member
# Description         : Table of all members of all households
#
# id                  : Unique id 
# first_name          : First name 
# last_name           : Surname 
# username            : Login id 
# password            : Authentication id 
# member_role_id      : Key into member_role
# household_id        : Key into household
#
CREATE TABLE member (
    id              INTEGER NOT NULL AUTO_INCREMENT,
    first_name      VARCHAR(100) NOT NULL,
    last_name       VARCHAR(100) NOT NULL,
    username        VARCHAR(100) NOT NULL,
    password        VARCHAR(255) NOT NULL,
    member_role_id  INTEGER,
    household_id    INTEGER,
PRIMARY KEY (id),
FOREIGN KEY (member_role_id) REFERENCES member_role (id),
FOREIGN KEY (household_id) REFERENCES household (id));
 
 
